# -*- coding: utf-8  -*-
"""Round-robin scheduling."""

# Copyright (C) 2015 Alexander Jones
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function, unicode_literals

import copy
import random

from . import NoMatchFound, ScheduleGenerationFailed
from .scheduler import Scheduler


class RoundRobinScheduler(Scheduler):

    """A standard round-robin scheduler."""

    def __init__(self, teams, meetings=0):
        """Constructor.

        @param teams: A list of teams or the number of teams
        @type teams: list or int
        @param meetings: The number of times teams meet each other
        @type meetings: int
        """
        if not isinstance(teams, list):
            teams = list(range(1, teams + 1))
        if len(teams) % 2 == 1:
            teams.append(None)
        self.teams = teams
        self.meetings = meetings

    @property
    def match_count(self):
        """The number of matches per round."""
        return int(len(self.teams) / 2)

    @property
    def round_count(self):
        """The number of rounds in a season."""
        return int((len(self.teams) - 1) * self.meetings)

    @property
    def home_teams(self):
        """The "home" teams from the previous schedule generation."""
        if hasattr(self, '_home_teams'):
            return tuple(self._home_teams)
        else:
            return ()

    def generate_matrix(self, home_teams=None):
        """Generate a schedule matrix for odd meeting counts."""
        team_count = len(self.teams)
        odd_team_count = None in self.teams
        if not home_teams:
            if odd_team_count:
                home_count = int((team_count - 1) / 2)
                homes = random.sample(range(team_count - 1), home_count)
                homes.append(team_count - 1)
            else:
                home_count = int(team_count / 2)
                homes = random.sample(range(team_count), home_count)
            self._home_teams = [self.teams[i] for i in homes]
        else:
            self._home_teams = home_teams
            homes = [self.teams.index(home) for home in home_teams]
        home_per_home = int(team_count / 2)
        home_per_away = int((team_count - 1) / 2)
        matrix = [[None] * team_count for __ in range(team_count)]
        for i in range(team_count - 1):
            home_team = i in homes
            home_count = (home_per_away
                          if not home_team or odd_team_count else home_per_home)
            for x in range(i):
                if matrix[i][x]:
                    home_count -= 1
            try:
                if odd_team_count:
                    home_opps = random.sample([x for x in range(i + 1,
                                                                team_count - 1)],
                                              home_count)
                    if home_team:
                        home_opps.append(team_count - 1)
                else:
                    home_opps = random.sample([x for x in range(i + 1,
                                                                team_count)],
                                              home_count)
                for opp in range(i + 1, team_count):
                    if opp in home_opps:
                        matrix[i][opp] = True
                        matrix[opp][i] = False
                    else:
                        matrix[i][opp] = False
                        matrix[opp][i] = True
            except ValueError:
                return self.generate_matrix(home_teams=home_teams)
        return matrix

    def generate_matches(self, home_teams=None):
        """Generate the matches for the season.

        @return: The matches to schedule
        @rtype: list
        """
        is_odd = self.meetings % 2 == 1
        if is_odd:
            evens = int((self.meetings - 1) / 2)
        else:
            evens = int(self.meetings / 2)

        # Even meetings
        if evens > 0:
            matches = [(team, opp)
                       for team in self.teams
                       for opp in self.teams
                       if team != opp] * evens
        else:
            matches = []
        # TODO: Odd meetings
        if is_odd:
            matrix = self.generate_matrix(home_teams=home_teams)
            odd_matches = []
            for team_idx in range(len(self.teams)):
                for opp_idx in range(team_idx + 1, len(self.teams)):
                    if matrix[team_idx][opp_idx]:
                        odd_matches.append((self.teams[team_idx],
                                            self.teams[opp_idx]))
                    else:
                        odd_matches.append((self.teams[opp_idx],
                                            self.teams[team_idx]))
            matches.extend(odd_matches)
        return matches

    def generate_round(self, matches):
        """Generate a round.

        @param matches: The generated matches
        @type matches: list
        @return: The generated round
        @rtype: list
        """
        round = []
        try:
            random.shuffle(matches)
            round.append(matches.pop(0))
            poss = copy.copy(matches)
            for __ in range(1, self.match_count):
                match = Scheduler.find_unique_match(round, poss)
                round.append(match)
                matches.remove(match)
            return round
        except NoMatchFound:
            matches.extend(round)
            return None

    def generate_schedule(self, try_once=False, home_teams=None):
        """Generate the schedule.

        @param try_once: Whether to only try once to generate a schedule
        @type try_once: bool
        @return: The generated schedule
        @rtype: list of lists of tuples
        @raise RuntimeError: Failed to create schedule within limits
        """
        rounds = []
        matches = self.generate_matches(home_teams=home_teams)

        for __ in range(self.round_count):
            for ___ in range(10):
                next_round = self.generate_round(matches)
                if next_round:
                    rounds.append(next_round)
                    break
            else:
                if not try_once:
                    return self.generate_schedule()
                else:
                    raise ScheduleGenerationFailed('Schedule generation failed.')

        return rounds


# Aliases for common meeting counts
class SingleRoundRobinScheduler(RoundRobinScheduler):

    """A standard single round-robin scheduler.

    This is an alias of RoundRobinScheduler, with meetings=1.
    """

    def __init__(self, teams):
        """Constructor.

        @param teams: A list of teams or the number of teams
        @type teams: list or int
        """
        super(SingleRoundRobinScheduler, self).__init__(teams, meetings=1)


class DoubleRoundRobinScheduler(RoundRobinScheduler):

    """A standard double round-robin scheduler.

    This is an alias of RoundRobinScheduler, with meetings=2.
    """

    def __init__(self, teams):
        """Constructor.

        @param teams: A list of teams or the number of teams
        @type teams: list or int
        """
        super(DoubleRoundRobinScheduler, self).__init__(teams, meetings=2)


class TripleRoundRobinScheduler(RoundRobinScheduler):

    """A standard triple round-robin scheduler.

    This is an alias of RoundRobinScheduler, with meetings=3.
    """

    def __init__(self, teams):
        """Constructor.

        @param teams: A list of teams or the number of teams
        @type teams: list or int
        """
        super(TripleRoundRobinScheduler, self).__init__(teams, meetings=3)


class QuadrupleRoundRobinScheduler(RoundRobinScheduler):

    """A standard quadruple round-robin scheduler.

    This is an alias of RoundRobinScheduler, with meetings=4.
    """

    def __init__(self, teams):
        """Constructor.

        @param teams: A list of teams or the number of teams
        @type teams: list or int
        """
        super(QuadrupleRoundRobinScheduler, self).__init__(teams, meetings=4)



# -*- coding: utf-8  -*-
"""Tests for round-robin schedulers."""

# Copyright (C) 2015 Alexander Jones
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import random

from . import TestCase, PY2, PY3

from competitions.scheduler import ScheduleGenerationFailed
from competitions.scheduler.roundrobin import (
    SingleRoundRobinScheduler,
    DoubleRoundRobinScheduler,
    TripleRoundRobinScheduler,
    QuadrupleRoundRobinScheduler
)


class TestSingleRoundRobin(TestCase):

    """Tests for single round-robin scheduling."""

    def test_match_generation(self):
        """Test single round-robin match generation."""
        # Four teams
        random.seed(1)
        scheduler = SingleRoundRobinScheduler(4)
        if PY2:
            expected_matches = [
                (1, 2), (3, 1), (1, 4),
                (2, 3), (4, 2), (3, 4)
            ]
        elif PY3:
            expected_matches = [
                (1, 2), (3, 1), (4, 1),
                (2, 3), (2, 4), (3, 4)
            ]
        matches = scheduler.generate_matches()
        self.assertCountEqual(expected_matches, matches,
                              ('Incorrect matches generated for single '
                               'round-robin schedule with four teams.'))
        # Three teams
        random.seed(1)
        scheduler = SingleRoundRobinScheduler(3)
        if PY2:
            expected_matches = [
                (2, 1), (1, 3), (3, 2),
                (1, None), (None, 2), (None, 3)
            ]
        elif PY3:
            expected_matches = [
                (1, 2), (3, 1), (2, 3),
                (1, None), (None, 2), (None, 3)
            ]
        matches = scheduler.generate_matches()
        self.assertCountEqual(expected_matches, matches,
                              ('Incorrect matches generated for single '
                               'round-robin schedule with three teams.'))
        # Eight teams
        random.seed(1)
        scheduler = SingleRoundRobinScheduler(8)
        if PY2:
            expected_matches = [
                (1, 2), (3, 1), (4, 1), (1, 5), (6, 1), (1, 7), (1, 8),
                (2, 3), (2, 4), (5, 2), (6, 2), (7, 2), (2, 8),
                (4, 3), (5, 3), (3, 6), (3, 7), (3, 8),
                (5, 4), (4, 6), (4, 7), (8, 4),
                (6, 5), (7, 5), (5, 8),
                (7, 6), (8, 6),
                (8, 7)
            ]
        elif PY3:
            expected_matches = [
                (1, 2), (3, 1), (4, 1), (1, 5), (1, 6), (1, 7), (8, 1),
                (3, 2), (2, 4), (5, 2), (2, 6), (7, 2), (2, 8),
                (3, 4), (5, 3), (6, 3), (3, 7), (8, 3),
                (4, 5), (4, 6), (7, 4), (8, 4),
                (5, 6), (5, 7), (8, 5),
                (6, 7), (6, 8),
                (7, 8)
            ]
        matches = scheduler.generate_matches()
        self.assertCountEqual(expected_matches, matches,
                              ('Incorrect matches generated for single '
                               'round-robin schedule with eight teams.'))

    def test_matrix_generation(self):
        """Test single round-robin matrix generation."""
        # Four teams
        random.seed(1)
        scheduler = SingleRoundRobinScheduler(4)
        if PY2:
            expected_matrix = [
                [None, True, False, True],
                [False, None, True, False],
                [True, False, None, True],
                [False, True, False, None]
            ]
        elif PY3:
            expected_matrix = [
                [None, True, False, False],
                [False, None, True, True],
                [True, False, None, True],
                [True, False, False, None]
            ]
        matrix = scheduler.generate_matrix()
        self.assertCountEqual(expected_matrix, matrix,
                              ('Incorrect matrix generated for single '
                               'round-robin schedule with four teams.'))
        # Three teams
        random.seed(1)
        scheduler = SingleRoundRobinScheduler(3)
        if PY2:
            expected_matrix = [
                [None, False, True, True],
                [True, None, False, False],
                [False, True, None, False],
                [False, True, True, None]
            ]
        elif PY3:
            expected_matrix = [
                [None, True, False, True],
                [False, None, True, False],
                [True, False, None, False],
                [False, True, True, None]
            ]
        matrix = scheduler.generate_matrix()
        self.assertCountEqual(expected_matrix, matrix,
                              ('Incorrect matrix generated for single '
                               'round-robin schedule with three teams.'))
        # Eight teams
        random.seed(1)
        scheduler = SingleRoundRobinScheduler(8)
        if PY2:
            expected_matrix = [
                [None, True, False, False, True, False, True, True],
                [False, None, True, True, False, False, False, True],
                [True, False, None, False, False, True, True, True],
                [True, False, True, None, False, True, True, False],
                [False, True, True, True, None, False, False, True],
                [True, True, False, False, True, None, False, False],
                [False, True, False, False, True, True, None, False],
                [False, False, False, True, False, True, True, None]
            ]
        elif PY3:
            expected_matrix = [
                [None, True, False, False, True, True, True, False],
                [False, None, False, True, False, True, False, True],
                [True, True, None, True, False, False, True, False],
                [True, False, False, None, True, True, False, False],
                [False, True, True, False, None, True, True, False],
                [False, False, True, False, False, None, True, True],
                [False, True, False, True, False, False, None, True],
                [True, False, True, True, True, False, False, None]
            ]
        matrix = scheduler.generate_matrix()
        self.assertCountEqual(expected_matrix, matrix,
                              ('Incorrect matrix generated for single '
                               'round-robin schedule with eight teams.'))

    def test_schedule_generation(self):
        """Test single round-robin schedule generation."""
        scheduler = SingleRoundRobinScheduler(8)
        # Failed attempt
        random.seed(17)
        self.assertRaises(ScheduleGenerationFailed,
                          scheduler.generate_schedule, try_once=True)
        # Successful attempt
        random.seed(4)
        if PY2:
            expected_schedule = [
                [(2, 5), (8, 6), (7, 1), (3, 4)],
                [(5, 8), (3, 7), (1, 2), (4, 6)],
                [(5, 4), (6, 3), (2, 7), (8, 1)],
                [(7, 8), (6, 5), (2, 4), (3, 1)],
                [(6, 2), (3, 8), (7, 5), (1, 4)],
                [(7, 6), (2, 3), (4, 8), (1, 5)],
                [(5, 3), (1, 6), (4, 7), (8, 2)]
            ]
        elif PY3:
            expected_schedule = [
                [(5, 7), (8, 4), (3, 6), (1, 2)],
                [(4, 2), (6, 8), (1, 5), (3, 7)],
                [(8, 1), (6, 4), (3, 5), (7, 2)],
                [(4, 1), (2, 3), (5, 8), (6, 7)],
                [(8, 3), (2, 6), (4, 5), (7, 1)],
                [(8, 2), (1, 3), (5, 6), (4, 7)],
                [(1, 6), (3, 4), (7, 8), (2, 5)]
            ]
        schedule = scheduler.generate_schedule(try_once=True)
        self.assertListEqual(expected_schedule, schedule,
                             ('Wrong schedule created for '
                              'single round-robin competition'))


class TestDoubleRoundRobin(TestCase):

    """Tests for double round-robin scheduling."""

    def test_match_generation(self):
        """Test double round-robin match generation."""
        # Four teams
        scheduler = DoubleRoundRobinScheduler(4)
        expected_matches = [
            (1, 2), (1, 3), (1, 4),
            (2, 1), (2, 3), (2, 4),
            (3, 1), (3, 2), (3, 4),
            (4, 1), (4, 2), (4, 3)
        ]
        matches = scheduler.generate_matches()
        self.assertCountEqual(expected_matches, matches,
                              ('Incorrect matches generated for double '
                               'round-robin schedule with four teams.'))
        # Three teams
        scheduler = DoubleRoundRobinScheduler(3)
        expected_matches = [
            (1, 2), (1, 3), (1, None),
            (2, 1), (2, 3), (2, None),
            (3, 1), (3, 2), (3, None),
            (None, 1), (None, 2), (None, 3)
        ]
        matches = scheduler.generate_matches()
        self.assertCountEqual(expected_matches, matches,
                              ('Incorrect matches generated for double '
                               'round-robin schedule with three teams.'))
        # Eight teams
        scheduler = DoubleRoundRobinScheduler(8)
        expected_matches = [
            (1, 2), (1, 3), (1, 4), (1, 5), (1, 6), (1, 7), (1, 8),
            (2, 1), (2, 3), (2, 4), (2, 5), (2, 6), (2, 7), (2, 8),
            (3, 1), (3, 2), (3, 4), (3, 5), (3, 6), (3, 7), (3, 8),
            (4, 1), (4, 2), (4, 3), (4, 5), (4, 6), (4, 7), (4, 8),
            (5, 1), (5, 2), (5, 3), (5, 4), (5, 6), (5, 7), (5, 8),
            (6, 1), (6, 2), (6, 3), (6, 4), (6, 5), (6, 7), (6, 8),
            (7, 1), (7, 2), (7, 3), (7, 4), (7, 5), (7, 6), (7, 8),
            (8, 1), (8, 2), (8, 3), (8, 4), (8, 5), (8, 6), (8, 7)
        ]
        matches = scheduler.generate_matches()
        self.assertCountEqual(expected_matches, matches,
                              ('Incorrect matches generated for double '
                               'round-robin schedule with eight teams.'))

    def test_schedule_generation(self):
        """Test double round-robin schedule generation."""
        scheduler = DoubleRoundRobinScheduler(8)
        # Failed attempt
        random.seed(5)
        self.assertRaises(ScheduleGenerationFailed,
                          scheduler.generate_schedule, try_once=True)
        # Successful attempt
        random.seed(2)
        if PY2:
            expected_schedule = [
                [(5, 6), (7, 4), (3, 8), (1, 2)],
                [(5, 1), (8, 4), (3, 6), (2, 7)],
                [(3, 1), (6, 5), (4, 8), (7, 2)],
                [(8, 5), (3, 2), (1, 4), (7, 6)],
                [(1, 8), (7, 3), (2, 6), (4, 5)],
                [(8, 1), (6, 3), (5, 7), (4, 2)],
                [(6, 1), (2, 5), (4, 3), (8, 7)],
                [(2, 4), (6, 8), (7, 5), (1, 3)],
                [(5, 4), (2, 1), (8, 6), (3, 7)],
                [(4, 7), (2, 8), (3, 5), (1, 6)],
                [(4, 6), (1, 7), (2, 3), (5, 8)],
                [(5, 3), (7, 8), (6, 2), (4, 1)],
                [(7, 1), (5, 2), (6, 4), (8, 3)],
                [(1, 5), (3, 4), (8, 2), (6, 7)]
            ]
        elif PY3:
            expected_schedule = [
                [(3, 2), (4, 1), (8, 5), (6, 7)],
                [(3, 8), (1, 6), (7, 5), (2, 4)],
                [(6, 8), (4, 7), (2, 3), (5, 1)],
                [(2, 1), (5, 7), (3, 6), (8, 4)],
                [(5, 4), (6, 2), (3, 7), (1, 8)],
                [(4, 3), (7, 2), (5, 6), (8, 1)],
                [(3, 1), (5, 2), (8, 6), (7, 4)],
                [(7, 1), (6, 4), (5, 3), (2, 8)],
                [(8, 7), (4, 6), (3, 5), (1, 2)],
                [(3, 4), (7, 8), (6, 1), (2, 5)],
                [(1, 4), (7, 3), (5, 8), (2, 6)],
                [(1, 3), (2, 7), (4, 8), (6, 5)],
                [(4, 5), (8, 2), (6, 3), (1, 7)],
                [(1, 5), (4, 2), (7, 6), (8, 3)]
            ]
        schedule = scheduler.generate_schedule(try_once=True)
        self.assertListEqual(expected_schedule, schedule,
                             ('Wrong schedule created for '
                              'double round-robin competition'))


class TestTripleRoundRobin(TestCase):

    """Tests for triple round-robin scheduling."""

    def test_match_generation(self):
        """Test triple round-robin match generation."""
        # Four teams
        random.seed(1)
        scheduler = TripleRoundRobinScheduler(4)
        if PY2:
            expected_matches = [
                (1, 2), (3, 1), (1, 4),
                (2, 3), (4, 2), (3, 4)
            ]
        elif PY3:
            expected_matches = [
                (1, 2), (3, 1), (4, 1),
                (2, 3), (2, 4), (3, 4)
            ]
        expected_matches.extend([
            (1, 2), (1, 3), (1, 4),
            (2, 1), (2, 3), (2, 4),
            (3, 1), (3, 2), (3, 4),
            (4, 1), (4, 2), (4, 3)
        ])
        matches = scheduler.generate_matches()
        self.assertCountEqual(expected_matches, matches,
                              ('Incorrect matches generated for triple '
                               'round-robin schedule with four teams.'))
        # Three teams
        random.seed(1)
        scheduler = TripleRoundRobinScheduler(3)
        if PY2:
            expected_matches = [
                (2, 1), (1, 3), (3, 2),
                (1, None), (None, 2), (None, 3)
            ]
        elif PY3:
            expected_matches = [
                (1, 2), (3, 1), (2, 3),
                (1, None), (None, 2), (None, 3)
            ]
        expected_matches.extend([
            (1, 2), (1, 3), (1, None),
            (2, 1), (2, 3), (2, None),
            (3, 1), (3, 2), (3, None),
            (None, 1), (None, 2), (None, 3)
        ])
        matches = scheduler.generate_matches()
        self.assertCountEqual(expected_matches, matches,
                              ('Incorrect matches generated for triple '
                               'round-robin schedule with three teams.'))
        # Eight teams
        random.seed(1)
        scheduler = TripleRoundRobinScheduler(8)
        if PY2:
            expected_matches = [
                (1, 2), (3, 1), (4, 1), (1, 5), (6, 1), (1, 7), (1, 8),
                (2, 3), (2, 4), (5, 2), (6, 2), (7, 2), (2, 8),
                (4, 3), (5, 3), (3, 6), (3, 7), (3, 8),
                (5, 4), (4, 6), (4, 7), (8, 4),
                (6, 5), (7, 5), (5, 8),
                (7, 6), (8, 6),
                (8, 7)
            ]
        elif PY3:
            expected_matches = [
                (1, 2), (3, 1), (4, 1), (1, 5), (1, 6), (1, 7), (8, 1),
                (3, 2), (2, 4), (5, 2), (2, 6), (7, 2), (2, 8),
                (3, 4), (5, 3), (6, 3), (3, 7), (8, 3),
                (4, 5), (4, 6), (7, 4), (8, 4),
                (5, 6), (5, 7), (8, 5),
                (6, 7), (6, 8),
                (7, 8)
            ]
        expected_matches.extend([
            (1, 2), (1, 3), (1, 4), (1, 5), (1, 6), (1, 7), (1, 8),
            (2, 1), (2, 3), (2, 4), (2, 5), (2, 6), (2, 7), (2, 8),
            (3, 1), (3, 2), (3, 4), (3, 5), (3, 6), (3, 7), (3, 8),
            (4, 1), (4, 2), (4, 3), (4, 5), (4, 6), (4, 7), (4, 8),
            (5, 1), (5, 2), (5, 3), (5, 4), (5, 6), (5, 7), (5, 8),
            (6, 1), (6, 2), (6, 3), (6, 4), (6, 5), (6, 7), (6, 8),
            (7, 1), (7, 2), (7, 3), (7, 4), (7, 5), (7, 6), (7, 8),
            (8, 1), (8, 2), (8, 3), (8, 4), (8, 5), (8, 6), (8, 7)
        ])
        matches = scheduler.generate_matches()
        self.assertCountEqual(expected_matches, matches,
                              ('Incorrect matches generated for triple '
                               'round-robin schedule with eight teams.'))

    def test_matrix_generation(self):
        """Test triple round-robin matrix generation."""
        # Four teams
        random.seed(1)
        scheduler = TripleRoundRobinScheduler(4)
        if PY2:
            expected_matrix = [
                [None, True, False, True],
                [False, None, True, False],
                [True, False, None, True],
                [False, True, False, None]
            ]
        elif PY3:
            expected_matrix = [
                [None, True, False, False],
                [False, None, True, True],
                [True, False, None, True],
                [True, False, False, None]
            ]
        matrix = scheduler.generate_matrix()
        self.assertCountEqual(expected_matrix, matrix,
                              ('Incorrect matrix generated for triple '
                               'round-robin schedule with four teams.'))
        # Three teams
        random.seed(1)
        scheduler = TripleRoundRobinScheduler(3)
        if PY2:
            expected_matrix = [
                [None, False, True, True],
                [True, None, False, False],
                [False, True, None, False],
                [False, True, True, None]
            ]
        elif PY3:
            expected_matrix = [
                [None, True, False, True],
                [False, None, True, False],
                [True, False, None, False],
                [False, True, True, None]
            ]
        matrix = scheduler.generate_matrix()
        self.assertCountEqual(expected_matrix, matrix,
                              ('Incorrect matrix generated for triple '
                               'round-robin schedule with three teams.'))
        # Eight teams
        random.seed(1)
        scheduler = TripleRoundRobinScheduler(8)
        if PY2:
            expected_matrix = [
                [None, True, False, False, True, False, True, True],
                [False, None, True, True, False, False, False, True],
                [True, False, None, False, False, True, True, True],
                [True, False, True, None, False, True, True, False],
                [False, True, True, True, None, False, False, True],
                [True, True, False, False, True, None, False, False],
                [False, True, False, False, True, True, None, False],
                [False, False, False, True, False, True, True, None]
            ]
        elif PY3:
            expected_matrix = [
                [None, True, False, False, True, True, True, False],
                [False, None, False, True, False, True, False, True],
                [True, True, None, True, False, False, True, False],
                [True, False, False, None, True, True, False, False],
                [False, True, True, False, None, True, True, False],
                [False, False, True, False, False, None, True, True],
                [False, True, False, True, False, False, None, True],
                [True, False, True, True, True, False, False, None]
            ]
        matrix = scheduler.generate_matrix()
        self.assertCountEqual(expected_matrix, matrix,
                              ('Incorrect matrix generated for triple '
                               'round-robin schedule with eight teams.'))

    def test_schedule_generation(self):
        """Test triple round-robin schedule generation."""
        scheduler = TripleRoundRobinScheduler(8)
        # Failed attempt
        random.seed(12)
        self.assertRaises(ScheduleGenerationFailed,
                          scheduler.generate_schedule, try_once=True)
        # Successful attempt
        random.seed(1)
        if PY2:
            expected_schedule = [
                [(6, 5), (1, 7), (3, 4), (2, 8)],
                [(7, 5), (4, 3), (8, 6), (1, 2)],
                [(1, 8), (5, 3), (7, 2), (6, 4)],
                [(5, 8), (3, 1), (4, 6), (7, 2)],
                [(5, 4), (8, 6), (1, 2), (3, 7)],
                [(6, 7), (3, 8), (2, 5), (4, 1)],
                [(4, 3), (5, 1), (2, 7), (6, 8)],
                [(7, 4), (2, 3), (6, 5), (1, 8)],
                [(5, 8), (1, 3), (4, 7), (6, 2)],
                [(4, 5), (7, 6), (3, 8), (2, 1)],
                [(3, 5), (7, 1), (8, 2), (4, 6)],
                [(1, 4), (6, 3), (7, 5), (2, 8)],
                [(7, 8), (1, 5), (4, 2), (3, 6)],
                [(7, 3), (5, 2), (8, 4), (1, 6)],
                [(3, 7), (4, 8), (2, 6), (1, 5)],
                [(2, 4), (5, 7), (6, 1), (8, 3)],
                [(3, 2), (8, 5), (4, 1), (7, 6)],
                [(5, 4), (8, 7), (6, 1), (2, 3)],
                [(5, 3), (6, 2), (4, 7), (8, 1)],
                [(5, 6), (8, 7), (3, 1), (2, 4)],
                [(8, 4), (5, 2), (1, 7), (3, 6)]
            ]
        elif PY3:
            expected_schedule = [
                [(6, 5), (4, 3), (8, 1), (7, 2)],
                [(7, 4), (6, 2), (8, 5), (3, 1)],
                [(1, 5), (3, 2), (7, 4), (8, 6)],
                [(7, 8), (6, 4), (3, 5), (1, 2)],
                [(4, 7), (2, 8), (3, 1), (5, 6)],
                [(3, 6), (8, 7), (4, 5), (1, 2)],
                [(5, 7), (3, 4), (2, 6), (8, 1)],
                [(2, 4), (6, 3), (5, 7), (1, 8)],
                [(8, 2), (7, 1), (5, 3), (4, 6)],
                [(7, 3), (1, 5), (8, 4), (2, 6)],
                [(6, 8), (7, 2), (5, 4), (1, 3)],
                [(1, 7), (8, 5), (2, 3), (4, 6)],
                [(2, 7), (1, 6), (4, 5), (8, 3)],
                [(6, 7), (5, 2), (8, 3), (4, 1)],
                [(6, 3), (4, 8), (7, 5), (2, 1)],
                [(1, 4), (3, 7), (5, 6), (2, 8)],
                [(2, 4), (6, 1), (5, 8), (3, 7)],
                [(5, 3), (1, 6), (4, 2), (7, 8)],
                [(5, 2), (3, 4), (6, 8), (1, 7)],
                [(3, 2), (8, 4), (6, 7), (5, 1)],
                [(7, 6), (4, 1), (3, 8), (2, 5)]
            ]
        schedule = scheduler.generate_schedule(try_once=True)
        self.assertListEqual(expected_schedule, schedule,
                             ('Wrong schedule created for '
                              'triple round-robin competition'))


class TestQuadrupleRoundRobin(TestCase):

    """Tests for quadruple round-robin scheduling."""

    def test_match_generation(self):
        """Test quadruple round-robin match generation."""
        # Four teams
        scheduler = QuadrupleRoundRobinScheduler(4)
        expected_matches = [
            (1, 2), (1, 3), (1, 4), (1, 2), (1, 3), (1, 4),
            (2, 1), (2, 3), (2, 4), (2, 1), (2, 3), (2, 4),
            (3, 1), (3, 2), (3, 4), (3, 1), (3, 2), (3, 4),
            (4, 1), (4, 2), (4, 3), (4, 1), (4, 2), (4, 3)
        ]
        matches = scheduler.generate_matches()
        self.assertCountEqual(expected_matches, matches,
                              ('Incorrect matches generated for quadruple '
                               'round-robin schedule with four teams.'))
        # Three teams
        scheduler = QuadrupleRoundRobinScheduler(3)
        expected_matches = [
            (1, 2), (1, 3), (1, None), (1, 2), (1, 3), (1, None),
            (2, 1), (2, 3), (2, None), (2, 1), (2, 3), (2, None),
            (3, 1), (3, 2), (3, None), (3, 1), (3, 2), (3, None),
            (None, 1), (None, 2), (None, 3), (None, 1), (None, 2), (None, 3)
        ]
        matches = scheduler.generate_matches()
        self.assertCountEqual(expected_matches, matches,
                              ('Incorrect matches generated for quadruple '
                               'round-robin schedule with three teams.'))
        # Eight teams
        scheduler = QuadrupleRoundRobinScheduler(8)
        expected_matches = [
            (1, 2), (1, 3), (1, 4), (1, 5), (1, 6), (1, 7), (1, 8),
            (1, 2), (1, 3), (1, 4), (1, 5), (1, 6), (1, 7), (1, 8),
            (2, 1), (2, 3), (2, 4), (2, 5), (2, 6), (2, 7), (2, 8),
            (2, 1), (2, 3), (2, 4), (2, 5), (2, 6), (2, 7), (2, 8),
            (3, 1), (3, 2), (3, 4), (3, 5), (3, 6), (3, 7), (3, 8),
            (3, 1), (3, 2), (3, 4), (3, 5), (3, 6), (3, 7), (3, 8),
            (4, 1), (4, 2), (4, 3), (4, 5), (4, 6), (4, 7), (4, 8),
            (4, 1), (4, 2), (4, 3), (4, 5), (4, 6), (4, 7), (4, 8),
            (5, 1), (5, 2), (5, 3), (5, 4), (5, 6), (5, 7), (5, 8),
            (5, 1), (5, 2), (5, 3), (5, 4), (5, 6), (5, 7), (5, 8),
            (6, 1), (6, 2), (6, 3), (6, 4), (6, 5), (6, 7), (6, 8),
            (6, 1), (6, 2), (6, 3), (6, 4), (6, 5), (6, 7), (6, 8),
            (7, 1), (7, 2), (7, 3), (7, 4), (7, 5), (7, 6), (7, 8),
            (7, 1), (7, 2), (7, 3), (7, 4), (7, 5), (7, 6), (7, 8),
            (8, 1), (8, 2), (8, 3), (8, 4), (8, 5), (8, 6), (8, 7),
            (8, 1), (8, 2), (8, 3), (8, 4), (8, 5), (8, 6), (8, 7)
        ]
        matches = scheduler.generate_matches()
        self.assertCountEqual(expected_matches, matches,
                              ('Incorrect matches generated for quadruple '
                               'round-robin schedule with eight teams.'))

    def test_schedule_generation(self):
        """Test quadruple round-robin schedule generation."""
        scheduler = QuadrupleRoundRobinScheduler(6)
        # Failed attempt
        random.seed(4)
        self.assertRaises(ScheduleGenerationFailed,
                          scheduler.generate_schedule, try_once=True)
        # Successful attempt
        random.seed(1)
        if PY2:
            expected_schedule = [
                [(5, 2), (6, 1), (4, 3)],
                [(1, 2), (3, 5), (4, 6)],
                [(5, 6), (3, 1), (4, 2)],
                [(4, 6), (1, 5), (2, 3)],
                [(2, 6), (5, 3), (4, 1)],
                [(2, 5), (3, 4), (6, 1)],
                [(4, 5), (2, 1), (3, 6)],
                [(2, 4), (1, 3), (6, 5)],
                [(3, 5), (2, 4), (1, 6)],
                [(2, 5), (4, 1), (3, 6)],
                [(5, 1), (6, 4), (3, 2)],
                [(4, 2), (5, 3), (1, 6)],
                [(1, 2), (4, 5), (6, 3)],
                [(2, 6), (1, 5), (4, 3)],
                [(6, 2), (3, 4), (5, 1)],
                [(5, 6), (3, 2), (1, 4)],
                [(5, 4), (6, 3), (2, 1)],
                [(5, 4), (1, 3), (6, 2)],
                [(3, 1), (5, 2), (6, 4)],
                [(2, 3), (6, 5), (1, 4)]
            ]
        elif PY3:
            expected_schedule = [
                [(6, 2), (5, 4), (3, 1)],
                [(5, 3), (2, 4), (1, 6)],
                [(1, 4), (6, 2), (5, 3)],
                [(6, 5), (1, 3), (4, 2)],
                [(1, 6), (3, 4), (5, 2)],
                [(1, 2), (5, 4), (3, 6)],
                [(2, 1), (6, 4), (3, 5)],
                [(4, 6), (5, 1), (3, 2)],
                [(5, 2), (3, 1), (4, 6)],
                [(2, 4), (5, 6), (1, 3)],
                [(5, 1), (2, 6), (3, 4)],
                [(6, 1), (2, 3), (4, 5)],
                [(4, 3), (2, 1), (6, 5)],
                [(3, 5), (6, 1), (4, 2)],
                [(1, 5), (2, 3), (6, 4)],
                [(1, 2), (6, 3), (4, 5)],
                [(2, 5), (4, 1), (6, 3)],
                [(1, 4), (3, 6), (2, 5)],
                [(3, 2), (4, 1), (5, 6)],
                [(1, 5), (2, 6), (4, 3)]
            ]
        schedule = scheduler.generate_schedule(try_once=True)
        self.assertListEqual(expected_schedule, schedule,
                             ('Wrong schedule created for '
                              'quadruple round-robin competition'))
